//    This file is part of BoobTracker.
//
//    BoobTracker is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    BoobTracker is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with BoobTracker.  If not, see <http://www.gnu.org/licenses/>.
//
//    You may contact the original author at : squalou.jenkins@gmail.com

package net.homelinux.pro.boobTraker;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.LinkedList;
import java.util.StringTokenizer;

import net.homelinux.pro.utils.FileUtils;
import net.homelinux.pro.utils.TimeUtils;
import android.os.Environment;

public class ListFeeds extends LinkedList<Feed> {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Boolean shrink=false; //if true : will only keep 10 latest elements

	public ListFeeds(Boolean shrink) {
		super();
		this.shrink=shrink;
	}

	public void loadDataFromFile() {

		if (this.isEmpty()){
	
			if (FileUtils.checkAvailabilityOfExternalStorage()) {
	
				File path = Environment.getExternalStorageDirectory();
				File file = new File(path, BoobConsts.DATA_FILE);
	
				addFeedsAfterReading(file);
			}
		}

	}



	private void addFeedsAfterReading(File file) {

		try {
			// prepare the file for reading
			BufferedInputStream buf = new BufferedInputStream(
					new FileInputStream(file));
			InputStreamReader inputreader = new InputStreamReader(buf);
			BufferedReader buffreader = new BufferedReader(inputreader);

			String line;

			// read every line of the file into the line-variable, on line
			// at the time
			while ((line = buffreader.readLine()) != null) {

				Feed feed = new Feed();

				StringTokenizer st1 = new StringTokenizer(line, ";");

				feed.setSide(st1.nextToken());
				try {
					feed.getDateStart().parse(TimeUtils.niceFileFormatDateToTime(st1.nextToken()));
				} catch (Exception e) {
					feed.getDateStart().setToNow();
				}
				try {
					feed.getDateEnd().parse(TimeUtils.niceFileFormatDateToTime(st1.nextToken()));
				} catch (Exception e) {
					feed.getDateEnd().setToNow();
				}
				try {
					feed.setDuration(st1.nextToken());

				} catch (Exception e) {
					feed.setDuration("");
				}
				try {
					feed.setPause(st1.nextToken());
				} catch (Exception e) {
					feed.setPause("");
				}
				
				feed.getDateStart().normalize(false);
				feed.getDateEnd().normalize(false);

				this.add(feed);
				//je sias pas si vaut mieux tout lire, tout ajouter, et shrinker à la fin
				//dans ce cas on ferait un super.add ici, et un shrink plus bas.
				//FIXME
				//super.add(feed);
			}
			
			buffreader.close();
			

		} catch (IOException e) {
			//Log.w("ExternalStorage", "Error accessing " + file, e);
			e.printStackTrace();

		}
		
		//FIXME
		//shrink();
		
	}

	/**
	 * adds a new feed to the list and also appends it to the file
	 * 
	 * @param feed
	 * @param shrink : if true, reduces the list to the 10 more recent elements only.
	 */
	protected void addAndStore(Feed feed) {

		if (! this.isEmpty()) computePauseTimeOfFeed(feed);
		
		if (FileUtils.checkAvailabilityOfExternalStorage()) {

			File path = Environment.getExternalStorageDirectory();
			File file = new File(path, BoobConsts.DATA_FILE);
			
			try {
				storeTo(file, feed);
			}catch(RuntimeException e){
				//Log.w("ExternalStorage", "Error storing to " + file, e);
				e.printStackTrace();

			}
		
		//once it is safely in the file, we add it to our list. This way we are sync
		
		this.add(feed);
		}

	}

	private void computePauseTimeOfFeed(Feed feed){
		feed.setPause(TimeUtils.millisToNiceDuration(TimeUtils.deltaMillis(feed.getDateStart(), this.getLast().getDateEnd())));
	}
	
	private void storeTo(File file, Feed feed) throws RuntimeException{
		
		if (! file.exists())
		{
			try {
				file.createNewFile();
			} catch (IOException e) {
				throw new RuntimeException("File Creation Error "+ file, e);
			}
		}
		
		try {
			BufferedOutputStream buf = new BufferedOutputStream(new FileOutputStream(file,true));
			OutputStreamWriter streamWriter=new OutputStreamWriter(buf);
		
			streamWriter.write(feed.toString()+"\n");
			
			streamWriter.close();
			
		}catch (FileNotFoundException e) {
			throw new RuntimeException("Not Found Error accessing " + file, e);
		}catch (SecurityException e) {
			throw new RuntimeException("Security Error accessing " + file, e);
		}catch (IOException e) {
			throw new RuntimeException("IO Error accessing " + file, e);
		}
		
	}

	
	public void flush(){
		if (FileUtils.checkAvailabilityOfExternalStorage()) {

			File path = Environment.getExternalStorageDirectory();
			File file = new File(path, BoobConsts.DATA_FILE);

			try {
				if (file.exists()) 
					file.delete();
			
				//this.clear(); not needed anymore, as listFeeds only keeps 10 last Feeds.
				//thiq way, screen won't loose datas.
				
			}catch(RuntimeException e){
				//Log.w("ExternalStorage", "Error deleting " + file, e);
				e.printStackTrace();
			}
			


		}

	}


	/**
	 * depending on the "shrink" attributes : if true, keeps only the 10 latests
	 * elements, in order to keep memory from being bloated. (we decided
	 * afterwards that the full log should be in file only and consulted
	 * separately)
	 */
	public boolean add(Feed feed) {

		super.add(feed);

		if (this.shrink) shrink();
		return true;
	}
	
	private void shrink(){
		if (this.size() > BoobConsts.BOOB_MAX_LISTFEED_SIZE) {
			int i = 0;
			// remonte le N derniers aux premières places
			for (i = 0; i < BoobConsts.BOOB_MAX_LISTFEED_SIZE; i++) {
				this.add(i, this.get(this.size() + i - BoobConsts.BOOB_MAX_LISTFEED_SIZE));
			}
			// vire les autres
			while (this.size() > BoobConsts.BOOB_MAX_LISTFEED_SIZE) {
				this.removeLast();
			}

		}
	}
	

}

