//    This file is part of BoobTracker.
//
//    BoobTracker is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    BoobTracker is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with BoobTracker.  If not, see <http://www.gnu.org/licenses/>.
//
//    You may contact the original author at : squalou.jenkins@gmail.com

package net.homelinux.pro.utils;

import android.text.format.Time;

/**
 * @author calou
 *
 */
public class TimeUtils {

	public TimeUtils() {

	}

	/**
	 * @param millis
	 * @return a string formatted as "HHhMMm" or "MMmSSs" or "SSs", converted from the long millis.
	 */
	public static String millisToNiceDuration(long millis) {
		
		String seconds= leadingZero(getSeconds(millis));
		String minutes=leadingZero(getMinutes(millis));
		String hours=leadingZero(getHours(millis));

		if (millis >= 3600000) {
			return hours + "h" + minutes;
		} else if (millis >= 60000) {
			return minutes + "m" + seconds;
		} else {
			return seconds + "s";
		}

		//ci dessous ça bugge sauvage dès que ça dépasse une heure ... étrange. Lié aux timezones ?
		
//		if (millis>=3600000){
//			SimpleDateFormat df = new SimpleDateFormat("HH'h'mm");
//			return df.format(new Date(millis));
//
//		}else if (millis>=60000){
//			SimpleDateFormat df = new SimpleDateFormat("mm'm'ss");
//			return df.format(new Date(millis));
//		}else{
//			SimpleDateFormat df = new SimpleDateFormat("ss's'");
//			return df.format(new Date(millis));
//		}
		
	}
	
	/** Rajoute autant de "0" devant machin pour que machin soit au moins long de 2 caractères.
	 * @param machin
	 * @return
	 */
	private static String leadingZero(String machin){
		while (machin.length()<2) {machin="0"+machin;};
		return machin;
	}

	
	/**
	 * @param millis
	 * @return nombre entier de minutes (0-59) d'après la durée donnée en millisecondes
	 */
	public static String getMinutes(long millis){
		return String.valueOf((int) Math.floor(millis/60000) % 60);
	}

	/**
	 * @param millis
	 * @return nombre entier d'heures d'après la durée donnée en millisecondes
	 */
	public static String getHours(long millis){
		return String.valueOf((int) Math.floor(millis/3600000));
	}
	
	/**
	 * @param millis
	 * @return nombre entier de secondes (0-59) d'après la durée donnée en millisecondes
	 */
	public static String getSeconds(long millis){
		return String.valueOf((int) Math.floor(millis/1000) % 60);
	}
	
	
	/**
	 * @param EndTime
	 * @param StartTime
	 * @param IgnoreDst : see Time.ToMillis
	 * @return milliseconds elapsed between EndTime and StartTime
	 */
	public static long deltaMillis(Time EndTime, Time StartTime, boolean IgnoreDst){
		return EndTime.toMillis(IgnoreDst)-StartTime.toMillis(IgnoreDst);
	}

	/**
	 * @param EndTime
	 * @param StartTime
	 * IgnoreDst = false : see Time.ToMillis
	 * @return milliseconds elapsed between EndTime and StartTime
	 */
	public static long deltaMillis(Time EndTime, Time StartTime){
		return deltaMillis(EndTime,StartTime,false);
	}	
	
	/**interprete la chaine issue du fichier de conf - au format de date idoine - et la transforme en chaine parsable par Time.parse();
	 * @param niceDate : chaine de la forme YYYY/mm/dd HH:MM:SS
	 * @return : a Time (01/01/1970 if parsing fails)
	 */
	public static String niceFileFormatDateToTime(String niceDate){
		return ((niceDate.replace(' ', 'T')).replaceAll("/", "")).replaceAll(":", "");
	}

}
